/*
 * LabelBox.cpp
 *
 *  Created on: Oct 27, 2008
 *      Author: dustin
 */

#include "LabelBox.h"
#include <gdkmm/drawable.h>
#include <gdkmm/general.h>  // for cairo helper functions
#include <iostream>
//#include <gtk/gtkwidget.h> //For GTK_IS_WIDGET()
#include <cstring>

namespace UI {

namespace Component {

LabelBox::LabelBox() :
	Glib::ObjectBase("LabelBox"),
	Gtk::Container(),
	m_child(0),
	cornerRadius(0),
	strokeWidth(2),
	padding_top(1),
	padding_bottom(1),
	padding_left(1),
	padding_right(1),
	fillColor("blue"),
	strokeColor("blue"),
	alpha(1)
{
	set_flags(Gtk::NO_WINDOW);
}

LabelBox::~LabelBox() {
}

void LabelBox::set_child_widgets(Gtk::Widget& child) {
	m_child = &child;

	m_child->set_parent(*this);
}

void LabelBox::setFillColor(const Gdk::Color & color) {
	fillColor = color;
}

void LabelBox::setBorderColor(const Gdk::Color & color) {
	strokeColor = color;
}

void LabelBox::setFillAlpha(double alpha) {
	this->alpha = alpha;
}

void LabelBox::setBorderWidth(double width) {
	strokeWidth = width;
}

void LabelBox::setCornerRadius(double radius) {
	cornerRadius = radius;
}

void LabelBox::setPadding(double padding) {
	setPadding(padding, padding, padding, padding);
}

void LabelBox::setPadding(double paddingTop, double paddingBottom,
		double paddingLeft, double paddingRight) {
	padding_top    = paddingTop;
	padding_bottom = paddingBottom;
	padding_left   = paddingLeft;
	padding_right  = paddingRight;
}


void LabelBox::on_size_request(Gtk::Requisition * requisition) {
	// Initialize the output parameter
	*requisition = Gtk::Requisition();

	Gtk::Requisition child_requisition = {0, 0};
	if (m_child && m_child->is_visible()) {
		child_requisition = m_child->size_request();
	}

	//See which one has the most width:
	int max_width = padding_left + (cornerRadius * 2)
					+ child_requisition.width + padding_right;

	//Add the heights together:
	int total_height = padding_top + (cornerRadius * 2)
					+ child_requisition.height + padding_bottom;

	//Request the width for this container based on the sizes requested by its
	//child widgets:
	requisition->height = total_height;
	requisition->width = max_width;

}

void LabelBox::on_size_allocate(Gtk::Allocation & allocation) {
	//Do something with the space that we have actually been given:
	//(We will not be given heights or widths less than we have requested, though
	//we might get more)

	//Use the offered allocation for this container:
	set_allocation(allocation);

	//Assign sign space to the child:
	Gtk::Allocation child_allocation;

	//Place the first child at the top-left,
	child_allocation.set_x( allocation.get_x() + padding_left + cornerRadius );
	child_allocation.set_y( allocation.get_y() + padding_top + cornerRadius );

	//Make it take up the full width available:
	child_allocation.set_width( allocation.get_width() - padding_left
								- padding_right - (cornerRadius * 2) );

	//Make it take up half the height available:
	child_allocation.set_height( allocation.get_height() - padding_top
								- padding_bottom - (cornerRadius * 2) );

	if(m_child && m_child->is_visible())
		m_child->size_allocate(child_allocation);

	if (m_refGdkWindow) {
		m_refGdkWindow->move_resize( allocation.get_x(), allocation.get_y(),
				allocation.get_width(), allocation.get_height() );
	}
}

void LabelBox::on_map() {
	//Call base class:
	Gtk::Container::on_map();
}

void LabelBox::on_unmap() {
	//Call base class:
	Gtk::Container::on_unmap();
}

void LabelBox::on_realize() {
	//Call base class:
	Gtk::Container::on_realize();

	ensure_style();

	if (!m_refGdkWindow) {
		//Create the GdkWindow:

		GdkWindowAttr attributes;
		memset(&attributes, 0, sizeof(attributes));

		Gtk::Allocation allocation = get_allocation();

		//Set initial position and size of the Gdk::Window:
		attributes.x = allocation.get_x();
		attributes.y = allocation.get_y();
		attributes.width = allocation.get_width();
		attributes.height = allocation.get_height();

		attributes.event_mask = get_events () | Gdk::EXPOSURE_MASK;
		attributes.window_type = GDK_WINDOW_CHILD;
		attributes.wclass = GDK_INPUT_OUTPUT;


		m_refGdkWindow = Gdk::Window::create(get_window() /* parent */, &attributes,
				GDK_WA_X | GDK_WA_Y);
		unset_flags(Gtk::NO_WINDOW);
		set_window(m_refGdkWindow);

		//set colors
		modify_bg(Gtk::STATE_NORMAL , Gdk::Color("white"));
		//modify_fg(Gtk::STATE_NORMAL , Gdk::Color("blue"));

		//make the widget receive expose events
		m_refGdkWindow->set_user_data(gobj());
	}
}

void LabelBox::on_unrealize() {
	m_refGdkWindow.clear();

	//Call base class:
	Gtk::Container::on_unrealize();
}

bool LabelBox::on_expose_event(GdkEventExpose* event) {

	if (m_refGdkWindow) {
		// Horizontal length of the line between the arched corners
		double hLength = (double)get_allocation().get_width()
						- padding_left - padding_right - (2 * cornerRadius);
		// Vertical length of the line between the arched corners
		double vLength = (double)get_allocation().get_height()
						- padding_top - padding_bottom - (2 * cornerRadius);

		Cairo::RefPtr<Cairo::Context> cr = m_refGdkWindow->create_cairo_context();
		if (event) {
			// clip to the area that needs to be re-exposed so we don't draw any
			// more than we need to.
			cr->rectangle(event->area.x, event->area.y,
			event->area.width, event->area.height);
			cr->clip();
		}

		// paint the background
		Gdk::Cairo::set_source_color(cr, get_style()->get_bg(Gtk::STATE_NORMAL));
		cr->paint();

		// draw the foreground
		cr->set_source_rgba(fillColor.get_red(), fillColor.get_green(), fillColor.get_blue(), alpha);
		cr->move_to(padding_left + cornerRadius, padding_top);
		cr->line_to(padding_left + cornerRadius + hLength, padding_top);
		cr->arc (padding_left + cornerRadius + hLength,
				padding_top + cornerRadius,
				cornerRadius, 1.5 * M_PI, 0 * M_PI);
		cr->line_to(padding_left + (2 * cornerRadius) + hLength,
				padding_top + cornerRadius + vLength);
		cr->arc (padding_left + cornerRadius + hLength,
				padding_top + cornerRadius + vLength,
				cornerRadius, 0 * M_PI, .5 * M_PI);
		cr->line_to(padding_left + cornerRadius,
				padding_top + (2 * cornerRadius) + vLength);
		cr->arc (padding_left + cornerRadius,
				padding_top + cornerRadius + vLength,
				cornerRadius, .5 * M_PI, 1 * M_PI);
		cr->line_to(padding_left, padding_top + cornerRadius);
		cr->arc (padding_left + cornerRadius,
				padding_top + cornerRadius,
				cornerRadius, 1 * M_PI, 1.5 * M_PI);
		cr->close_path();
		cr->fill();

		cr->set_source_rgb(strokeColor.get_red(), strokeColor.get_green(), strokeColor.get_blue());
		cr->set_line_width(strokeWidth);
		cr->move_to(padding_left + cornerRadius, padding_top);
		cr->line_to(padding_left + cornerRadius + hLength, padding_top);
		cr->arc (padding_left + cornerRadius + hLength,
				padding_top + cornerRadius,
				cornerRadius, 1.5 * M_PI, 0 * M_PI);
		cr->line_to(padding_left + (2 * cornerRadius) + hLength,
				padding_top + cornerRadius + vLength);
		cr->arc (padding_left + cornerRadius + hLength,
				padding_top + cornerRadius + vLength,
				cornerRadius, 0 * M_PI, .5 * M_PI);
		cr->line_to(padding_left + cornerRadius,
				padding_top + (2 * cornerRadius) + vLength);
		cr->arc (padding_left + cornerRadius,
				padding_top + cornerRadius + vLength,
				cornerRadius, .5 * M_PI, 1 * M_PI);
		cr->line_to(padding_left, padding_top + cornerRadius);
		cr->arc (padding_left + cornerRadius,
				padding_top + cornerRadius,
				cornerRadius, 1 * M_PI, 1.5 * M_PI);
		cr->close_path();
		cr->stroke();
	}

	//Call base class:
	return Gtk::Container::on_expose_event(event);

}

void LabelBox::forall_vfunc(gboolean /* include_internals */,
		GtkCallback callback, gpointer callback_data)
{
	if (m_child)
		callback(m_child->gobj(), callback_data);
}

void LabelBox::on_add(Gtk::Widget* child) {
	if (!m_child) {
		m_child = child;
		m_child->set_parent(*this);
	}
}

void LabelBox::on_remove(Gtk::Widget* child) {
	if (child) {

		const bool visible = child->is_visible();
		bool found = false;

		if (child == m_child) {
			m_child = 0;
			found = true;
		}

		if (found) {
			child->unparent();

			if(visible)
			queue_resize();
		}
	}
}

GtkType LabelBox::child_type_vfunc() const {
	//If there is still space for one widget, then report the type of widget that
	//may be added.
	if (!m_child)
		return Gtk::Widget::get_type();
	else {
		//No more widgets may be added.
		return G_TYPE_NONE;
	}
}


}

}
